%{
/* mapparser.y
 *
 * Yacc script for parsing map information
 */
#include "sim.h"
#include "server.h"
%}

%union {
	double num;
	char string[256];
	}
%token <num> NUM
%token <string> STRING
%token END,NAME,CREATOR,PASSWORD,SAFENODE,NODESFILE,PATHSFILE,SHAPESFILE
%token OBJECTSFILE,PERSONSFILE,PLAYERSFILE,ACTIONSFILE,EXPECTEDHOST,
%token EXPECTEDPORT
%%
input		:	mapattribs END
			{ return 0;
			}
		;

mapattribs	:	mapattribs mapattrib
		|
		;

mapattrib	:	NAME STRING
			{ mp->name = strdupe($2);
			}
		|	CREATOR STRING
			{ mp->creator = strdupe($2);
			}
		|	PASSWORD STRING
			{ mp->password = strdupe($2);
			}
		|	SAFENODE safenodes
		|	NODESFILE STRING
			{ mp->ndfile = strdupe($2);
			}
		|	PATHSFILE STRING
			{ mp->ptfile = strdupe($2);
			}
		|	SHAPESFILE STRING
			{ mp->shfile = strdupe($2);
			}
		|	OBJECTSFILE STRING
			{ mp->obfile = strdupe($2);
			}
		|	PERSONSFILE STRING
			{ mp->prfile = strdupe($2);
			}
		|	PLAYERSFILE STRING
			{ mp->plfile = strdupe($2);
			}
		|	ACTIONSFILE STRING
			{ mp->acfile = strdupe($2);
			}
		|	EXPECTEDHOST STRING
			{ mp->expectedhost = strdupe($2);
			}
		|	EXPECTEDPORT NUM
			{ mp->expectedport = $2;
			}
		;

safenodes	:	safenode safenodes
		|
		;

safenode	:	STRING
			{ mp->safenode[mp->safenodec++] = strdupe($1);
			}
		;
%%
static reserved rwtab[] = {	"end",END,"name",NAME,"creator",CREATOR,
				"password",PASSWORD,"safenode",SAFENODE,
				"nodesfile",NODESFILE,"pathsfile",PATHSFILE,
				"shapesfile",SHAPESFILE,
				"objectsfile",OBJECTSFILE,
				"personsfile",PERSONSFILE,
				"playersfile",PLAYERSFILE,
				"actionsfile",ACTIONSFILE,
				"expectedhost",EXPECTEDHOST,
				"expectedport",EXPECTEDPORT };

#define error yyerror
#define lex yylex
#define lval yylval

#include "lexer.h"

/* parsemap
 * Fills in a map structure
 */
int parse_map(int filed)
{
init_lexer(filed);
return yyparse();
}


/* write_map
 * Output the map to a file descriptor.
 */
void write_map(int fd, int tl)
{
int i;

fdprintf(fd,"%sname         \"%s\"\n",tabs(tl),mp->name);
fdprintf(fd,"%screator      \"%s\"\n",tabs(tl),mp->creator);
fdprintf(fd,"%spassword     \"%s\"\n",tabs(tl),mp->password);
fdprintf(fd,"%ssafenode     ",tabs(tl));
for(i=0; i<mp->safenodec; i++)
	fdprintf(fd,"\"%s\" ",mp->safenode[i]);
fdprintf(fd,"\n");
if (mp->expectedhost)
	fdprintf(fd,"%sexpectedhost \"%s\"\n",tabs(tl),mp->expectedhost);
if (mp->expectedport)
	fdprintf(fd,"%sexpectedport %d\n",tabs(tl),mp->expectedport);
fdprintf(fd,"%snodesfile    \"%s\"\n",tabs(tl),mp->ndfile);
fdprintf(fd,"%spathsfile    \"%s\"\n",tabs(tl),mp->ptfile);
fdprintf(fd,"%sshapesfile   \"%s\"\n",tabs(tl),mp->shfile);
fdprintf(fd,"%sobjectsfile  \"%s\"\n",tabs(tl),mp->obfile);
fdprintf(fd,"%spersonsfile  \"%s\"\n",tabs(tl),mp->prfile);
fdprintf(fd,"%splayersfile  \"%s\"\n",tabs(tl),mp->plfile);
fdprintf(fd,"%sactionsfile  \"%s\"\n",tabs(tl),mp->acfile);
fdprintf(fd,"end\n");
}

